﻿namespace WindowsFormsApplication2
{
	using System;
	using System.Diagnostics;
	using System.Runtime.InteropServices;
	using System.Threading;
	using EXCEPINFO = System.Runtime.InteropServices.ComTypes.EXCEPINFO;

	[ComVisible(true)]
	public class MyObject
	{
		public string Value
		{
			get { return Guid.NewGuid().ToString(); }
		}
	}

	[ComVisible(true)]
	[ClassInterface(ClassInterfaceType.AutoDispatch)]
	public class Interpreter :
		IActiveScriptSite,
		IDisposable
	{
		private string _result;

		public string RunVB(
			string runString)
		{
			var sw1 = new Stopwatch();
			sw1.Start();

			var engineObject = new VBScriptEngine();
			var engine = (IActiveScript)engineObject;

			// Query for interface.
			var parser = new ActiveScriptParseWrapper(engine);
			parser.InitNew();

			engine.SetScriptSite(this);

			// --

			engine.SetScriptState(ScriptState.Initialized);

			engine.AddNamedItem(
				@"Context",
				ScriptItem.IsVisible | ScriptItem.GlobalMembers);

			object result;
			EXCEPINFO ei;

			parser.ParseScriptText(
				runString,
				null,
				null,
				null,
				IntPtr.Zero,
				0,
				ScriptText.IsVisible,
				out result,
				out ei);

			engine.SetScriptState(ScriptState.Started);
			engine.SetScriptState(ScriptState.Connected);

			// Here, the item is run.

			engine.Close();

			parser.ReleaseUweKeim();
			Marshal.ReleaseComObject(engine);
			Marshal.ReleaseComObject(engineObject);

			sw1.Stop();
			return sw1.ElapsedMilliseconds.ToString();

			return _result;
		}

		public string RunJS(
			string runString)
		{
			var sw1 = new Stopwatch();
			sw1.Start();

			var engineObject = new JScriptEngine();
			var engine = (IActiveScript)engineObject;

			// Query for interface.
			var parser = new ActiveScriptParseWrapper(engine);
			parser.InitNew();

			engine.SetScriptSite(this);

			// --

			engine.SetScriptState(ScriptState.Initialized);

			engine.AddNamedItem(
				@"Context",
				ScriptItem.IsVisible | ScriptItem.GlobalMembers);

			object result;
			EXCEPINFO ei;

			parser.ParseScriptText(
				runString,
				null,
				null,
				null,
				IntPtr.Zero,
				0,
				ScriptText.IsVisible,
				out result,
				out ei);

			engine.SetScriptState(ScriptState.Started);
			engine.SetScriptState(ScriptState.Connected);

			// Here, the item is run.

			engine.Close();

			parser.ReleaseUweKeim();
			Marshal.ReleaseComObject(engine);
			Marshal.ReleaseComObject(engineObject);

			sw1.Stop();
			return sw1.ElapsedMilliseconds.ToString();

			return _result;
		}

		public MyObject CreateMyObject()
		{
			return new MyObject();
		}

		public void WriteLine(string text)
		{
			_result += text+Environment.NewLine;
		}

		#region Implementation of IActiveScriptSite

		void IActiveScriptSite.GetLCID(out int lcid)
		{
			lcid = Thread.CurrentThread.CurrentUICulture.LCID;
		}

		void IActiveScriptSite.GetItemInfo(string name, ScriptInfo returnMask, object[] item, IntPtr[] typeInfo)
		{
			if (name == null)
			{
				throw new ArgumentNullException(@"name");
			}

			if (name != @"Context")
			{
				throw new COMException(null, (int)HRESULT.TYPE_E_ELEMENTNOTFOUND);
			}

			if ((returnMask & ScriptInfo.IUnknown) != 0)
			{
				if (item == null)
				{
					throw new ArgumentNullException(@"item");
				}

				item[0] = this;
			}

			if ((returnMask & ScriptInfo.ITypeInfo) != 0)
			{
				if (typeInfo == null)
				{
					throw new ArgumentNullException(@"typeInfo");
				}

				typeInfo[0] = IntPtr.Zero;
			}
		}

		void IActiveScriptSite.GetDocVersionString(out string version)
		{
			throw new NotImplementedException();
		}

		void IActiveScriptSite.OnScriptTerminate(object result, EXCEPINFO exceptionInfo)
		{
		}

		void IActiveScriptSite.OnStateChange(ScriptState scriptState)
		{
		}

		void IActiveScriptSite.OnScriptError(IActiveScriptError scriptError)
		{
		}

		void IActiveScriptSite.OnEnterScript()
		{
		}

		void IActiveScriptSite.OnLeaveScript()
		{
		}

		#endregion

		public void Dispose()
		{
		}
	}
}